@@ -1,29 +1,31 @@
 # -*- coding: utf-8 -*-
-from PyQt4.QtGui import *
-from PyQt4.QtCore import *
-import sys, resource, os, audioinfo
-from puddleobjects import (ListButtons, OKCancel, HeaderSetting, ListBox,
-    PuddleConfig, savewinsize, winsettings, encode_fn, decode_fn)
+import os
+
+from PyQt6.QtCore import Qt
+from PyQt6.QtWidgets import QHBoxLayout, QListWidgetItem, QWidget
+
+from . import audioinfo
+from .constants import CONFIGDIR
+from .puddleobjects import (ListButtons, ListBox)
+
 
 def load_genres(filepath=None):
     if not filepath:
-        cparser = PuddleConfig()
-        filepath = os.path.join(cparser.savedir, 'genres')
+        filepath = os.path.join(CONFIGDIR, 'genres')
     try:
-        return [unicode(z.strip(), 'utf8') for z in
-            open(filepath, 'r').readlines()]
+        return [x.strip() for x in open(filepath, 'r').readlines() if x.strip()]
     except (IOError, OSError):
         return audioinfo.GENRES[::]
 
+
 def save_genres(genres, filepath=None):
-    cparser = PuddleConfig()
     if not filepath:
-        filepath = os.path.join(cparser.savedir, 'genres')
+        filepath = os.path.join(CONFIGDIR, 'genres')
     f = open(filepath, 'w')
-    text = '\n'.join([z.encode('utf8') for z in genres])
-    f.write(text)
+    f.write('\n'.join(genres))
     f.close()
 
+
 class Genres(QWidget):
     def __init__(self, parent=None, status=None):
         QWidget.__init__(self, parent)
@@ -35,18 +37,18 @@ class Genres(QWidget):
             genres = status['genres']
 
         self.listbox = ListBox()
-        self._itemflags = Qt.ItemIsSelectable | Qt.ItemIsEditable | Qt.ItemIsEnabled
+        self._itemflags = Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEditable | Qt.ItemFlag.ItemIsEnabled
         [self.listbox.addItem(self._createItem(z)) for z in genres]
 
         buttons = ListButtons()
         self.listbox.connectToListButtons(buttons)
         self.listbox.setAutoScroll(False)
 
-        self.connect(buttons, SIGNAL('add'), self.add)
-        self.connect(buttons, SIGNAL('edit'), self.edit)
+        buttons.add.connect(self.add)
+        buttons.edit.connect(self.edit)
 
         hbox = QHBoxLayout()
-        hbox.addWidget(self.listbox,1)
+        hbox.addWidget(self.listbox, 1)
         hbox.addLayout(buttons, 0)
         self.setLayout(hbox)
 
@@ -71,6 +73,6 @@ class Genres(QWidget):
 
     def applySettings(self, control=None):
         item = self.listbox.item
-        genres = [unicode(item(row).text()) for row in
-            xrange(self.listbox.count())]
-        self._status['genres'] = genres
\ No newline at end of file
+        genres = [str(item(row).text()) for row in
+                  range(self.listbox.count())]
+        self._status['genres'] = genres
