@@ -1,22 +1,24 @@
 # -*- coding: utf-8 -*-
-from puddleobjects import PuddleConfig, get_icon
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-import sys, pdb, resource,os
-from constants import SAVEDIR, DATADIR
-import StringIO
-from util import open_resourcefile
-from puddlestuff.translations import translate
+import os
+import sys
 
-__version__ = 29
+from PyQt6.QtGui import QAction
+from PyQt6.QtWidgets import QApplication, QMainWindow, QMenu, QMenuBar, QToolBar
+
+from .constants import CONFIGDIR
+from .puddleobjects import PuddleConfig, get_icon, open_resourcefile
+from .translations import translate
+
+__version__ = 33
 
 files = [open_resourcefile(filename)
-    for filename in [':/caseconversion.action', ':/standard.action']]
+         for filename in ['data:./caseconversion.action', 'data:./standard.action']]
 
 SEPARATOR = 'separator'
 ALWAYS = 'always'
-menu_path = os.path.join(SAVEDIR, 'menus')
-shortcut_path = os.path.join(SAVEDIR, 'shortcuts')
+menu_path = os.path.join(CONFIGDIR, 'menus')
+shortcut_path = os.path.join(CONFIGDIR, 'shortcuts')
+
 
 def create_file(path, resource):
     text = open_resourcefile(resource).read()
@@ -24,18 +26,23 @@ def create_file(path, resource):
     f.write(text)
     f.close()
 
-def check_file(path, resource):
+
+def check_file(path: str, resource: str) -> None:
+    """Check version of config file at path, and update with resource if needed."""
     if not os.path.exists(path):
         create_file(path, resource)
     else:
         cparser = PuddleConfig(path)
         version = cparser.get('info', 'version', 0)
         if version < __version__:
+            print(f'Replacing version {version} config file {path} with newer version')
             create_file(path, resource)
 
+
 def create_files():
-    check_file(menu_path, ':/menus')
-    check_file(shortcut_path, ':/shortcuts')
+    check_file(menu_path, 'data:./menus')
+    check_file(shortcut_path, 'data:./shortcuts')
+
 
 def get_menus(section, filepath=None):
     cparser = PuddleConfig()
@@ -43,13 +50,14 @@ def get_menus(section, filepath=None):
         filepath = menu_path
     cparser.filename = filepath
     menus = []
-    settings = cparser.settings
+    settings = cparser.data
     temp = settings[section]
     menus = [(z, temp[z]) for z in settings[section + 'attrs']['order']]
     return menus
 
+
 def menubar(menus, actions):
-    texts = [unicode(action.text()) for action in actions]
+    texts = [str(action.text()) for action in actions]
 
     menubar = QMenuBar()
     winmenu = None
@@ -57,7 +65,7 @@ def menubar(menus, actions):
     for title, actionlist in menus:
         menu = menubar.addMenu(translate("Menus", title))
         _menus[title] = [menu]
-        if title == u'&Windows':
+        if title == '&Windows':
             winmenu = menu
             tr_section = 'Dialogs'
         else:
@@ -72,6 +80,7 @@ def menubar(menus, actions):
                 menu.addSeparator()
     return menubar, winmenu, _menus
 
+
 def context_menu(section, actions, filepath=None):
     cparser = PuddleConfig(filepath)
     if not filepath:
@@ -80,7 +89,7 @@ def context_menu(section, actions, filepath=None):
     order = [translate('Menus', z) for z in cparser.get(section, 'order', [])]
     if not order:
         return
-    texts = [unicode(action.text()) for action in actions]
+    texts = [str(action.text()) for action in actions]
     menu = QMenu()
     for action in order:
         if action in texts:
@@ -89,10 +98,11 @@ def context_menu(section, actions, filepath=None):
             menu.addSeparator()
     return menu
 
+
 def toolbar(groups, actions, controls=None):
-    texts = [unicode(action.text()) for action in actions]
+    texts = [str(action.text()) for action in actions]
     if controls:
-        controls = dict([('widget-' + z, v) for z,v in controls.items()])
+        controls = dict([('widget-' + z, v) for z, v in controls.items()])
     toolbar = QToolBar('Toolbar')
     for name, actionlist in groups:
         for action in actionlist:
@@ -103,10 +113,11 @@ def toolbar(groups, actions, controls=None):
         toolbar.addSeparator()
     return toolbar
 
-def create_action(win, name, control, command, icon = None, enabled=ALWAYS,
-    tooltip=None, shortcut=None, status=None, togglecheck=None,
-    checkstate=None, icon_name=None):
-    if icon:
+
+def create_action(win, name, control, command, icon=None, enabled=ALWAYS,
+                  tooltip=None, shortcut=None, status=None, togglecheck=None,
+                  checkstate=None, icon_name=None):
+    if icon or icon_name:
         action = QAction(get_icon(icon_name, icon), name, win)
     else:
         action = QAction(name, win)
@@ -120,7 +131,7 @@ def create_action(win, name, control, command, icon = None, enabled=ALWAYS,
 
     if tooltip:
         action.setToolTip(translate('Menus', tooltip))
-    
+
     if togglecheck is not None:
         action.setCheckable(True)
         checked = int(checkstate)
@@ -134,23 +145,25 @@ def create_action(win, name, control, command, icon = None, enabled=ALWAYS,
 
     return action
 
+
 def get_actions(parent, filepath=None):
     cparser = PuddleConfig()
     if not filepath:
         filepath = shortcut_path
     cparser.filename = filepath
-    setting = cparser.settings
+    setting = cparser.data
     actions = []
     for section in cparser.sections():
         if section.startswith('shortcut'):
-            values = dict([(str(k), v) for k,v in  setting[section].items()])
+            values = dict([(str(k), v) for k, v in setting[section].items()])
             actions.append(create_action(parent, **values))
     return actions
 
+
 if __name__ == '__main__':
     app = QApplication(sys.argv)
     win = QMainWindow()
     win.toolbar = win.addToolBar('toolbar')
     loadShortCuts()
     win.show()
-    app.exec_()
+    app.exec()
