@@ -1,13 +1,17 @@
-# -*- coding: utf-8 -*-
-import os, traceback, sys
-from puddlestuff.puddleobjects import PuddleConfig, winsettings
-from puddlestuff.constants import (FUNCTIONS, TAGSOURCE, SAVEDIR,
-    DIALOGS, MUSICLIBS, MODULES)
-from os.path import splitext, exists
+import logging
+import os
+import sys
+from os.path import exists
+from importlib import import_module
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from puddlestuff.translations import translate
+from PyQt6.QtCore import Qt
+from PyQt6.QtWidgets import QApplication, QDialog, QHBoxLayout, QLabel, QListWidget, QListWidgetItem, QVBoxLayout
+from configobj import ConfigObj
+
+from .constants import (FUNCTIONS, TAGSOURCE, SAVEDIR, PLUGINDIR,
+                        DIALOGS, MUSICLIBS, MODULES, FUNCTIONS_NO_PREVIEW)
+from .puddleobjects import PuddleConfig, winsettings
+from .translations import translate
 
 NAME = 'name'
 AUTHOR = 'author'
@@ -17,11 +21,12 @@ VERSION = 'version'
 INFO_SECTION = 'info'
 MODULE_NAME = 'module'
 
-PLUGIN_DIRS = [os.path.join(SAVEDIR, u'plugins'),
-    os.path.join(os.path.dirname(__file__), 'plugins')]
+PLUGIN_DIRS = [PLUGINDIR,
+               os.path.join(os.path.dirname(__file__), 'plugins')]
 
 PROPERTIES = [NAME, AUTHOR, DESC, PT_VERSION, VERSION]
 
+
 def get_plugins(plugindir):
     if not os.path.exists(plugindir):
         os.makedirs(plugindir)
@@ -30,15 +35,17 @@ def get_plugins(plugindir):
         info_path = os.path.join(plugindir, module, 'info')
         if not exists(info_path):
             continue
-        cparser = PuddleConfig(info_path)
-        values = [cparser.get(INFO_SECTION, prop, '') for prop in PROPERTIES]
-        if len(filter(None, values)) < len(PROPERTIES):
+        config_data = dict(ConfigObj(info_path))
+        values = [config_data.get(INFO_SECTION, {}).get(prop, '') for prop in PROPERTIES]
+
+        if len([_f for _f in values if _f]) < len(PROPERTIES):
             continue
-        d = dict(zip(PROPERTIES, values))
+        d = dict(list(zip(PROPERTIES, values)))
         d[MODULE_NAME] = module
         infos.append(d)
     return infos
 
+
 def load_plugins(plugins=None, parent=None):
     [sys.path.insert(0, d) for d in PLUGIN_DIRS]
     cparser = PuddleConfig()
@@ -48,114 +55,114 @@ def load_plugins(plugins=None, parent=None):
     dialogs = []
     musiclibs = []
     modules = []
-    
+    functions_no_preview = []
+
     join = os.path.join
     if plugins is None:
         plugins = []
         [plugins.extend(get_plugins(d)) for d in PLUGIN_DIRS]
 
-    plugins.sort(key=lambda d: d.get(NAME, u''))
+    plugins.sort(key=lambda d: d.get(NAME, ''))
+
+    plugin_actions = {
+        'functions': functions.update,
+        'functions_no_preview':  functions_no_preview.extend,
+        'tagsources':  tagsources.extend,
+        'dialogs':  dialogs.extend,
+        'musiclibs':  musiclibs.extend
+        }
 
     for plugin in plugins:
-        if plugin[MODULE_NAME] not in to_load:
-            continue
-        try:
-            module = __import__(plugin[MODULE_NAME])
-        except:
-            print u'Failed to load plugin: ', plugin['name']
-            traceback.print_exc()
-            continue
-        if hasattr(module, 'functions'):
-            functions.update(module.functions)
+        if plugin[MODULE_NAME] in to_load:
+            try:
+                module = import_module('puddlestuff.plugins.' + plugin[MODULE_NAME])
+                for attribute, action in plugin_actions.items():
+                    if hasattr(module, attribute):
+                        action(getattr(module, attribute))
+                modules.append(module)
+            except Exception as e:
+                logging.exception(f"Failed to load plugin {plugin[NAME]}; error={e}")                
 
-        if hasattr(module, 'tagsources'):
-            tagsources.extend(module.tagsources)
-
-        if hasattr(module, 'dialogs'):
-            dialogs.extend(module.dialogs)
-
-        if hasattr(module, 'musiclibs'):
-            musiclibs.extend(module.musiclibs)
-
-        modules.append(module)
-        
     for d in PLUGIN_DIRS:
-        del(sys.path[0])
+        del (sys.path[0])
 
     return {FUNCTIONS: functions, TAGSOURCE: tagsources, DIALOGS: dialogs,
-        MUSICLIBS: musiclibs, MODULES: modules}
+            MUSICLIBS: musiclibs, MODULES: modules, FUNCTIONS_NO_PREVIEW: functions_no_preview}
+
 
 class InfoWidget(QLabel):
     def __init__(self, info=None, parent=None):
         super(InfoWidget, self).__init__(parent)
-        self.setAlignment(Qt.AlignLeft | Qt.AlignTop)
+        self.setAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignTop)
         self.setWordWrap(True)
         if info:
             self.changeInfo(info)
-    
+
     def changeInfo(self, info):
         labels = [translate('Plugin Settings', 'Name'),
-            translate('Plugin Settings', 'Author'),
-            translate('Plugin Settings', 'Description'),
-            translate('Plugin Settings', 'Version')]
+                  translate('Plugin Settings', 'Author'),
+                  translate('Plugin Settings', 'Description'),
+                  translate('Plugin Settings', 'Version')]
         properties = [NAME, AUTHOR, DESC, VERSION]
-        
-        text = u'<br />'.join([u'<b>%s:</b> %s' % (disp, info[prop]) for 
-            disp, prop in zip(labels, properties)])
+
+        text = '<br />'.join(['<b>%s:</b> %s' % (disp, info[prop]) for
+                              disp, prop in zip(labels, properties)])
         self.setText(text)
 
+
 class PluginConfig(QDialog):
-    def __init__(self, parent = None):
+    def __init__(self, parent=None):
         super(PluginConfig, self).__init__(parent)
         winsettings('pluginconfig', self)
         self._listbox = QListWidget()
         info_display = InfoWidget()
-        
+
         hbox = QHBoxLayout()
         hbox.addWidget(self._listbox, 0)
         hbox.addWidget(info_display, 1)
-        
+
         vbox = QVBoxLayout()
         vbox.addLayout(hbox)
         vbox.addWidget(
             QLabel(translate("Plugin Settings",
-                '<b>Loading/unloading plugins requires a restart.</b>')))
+                             '<b>Loading/unloading plugins requires a restart.</b>')))
         self.setLayout(vbox)
 
         plugins = []
         [plugins.extend(get_plugins(d)) for d in PLUGIN_DIRS]
-        
+
         cparser = PuddleConfig()
         to_load = cparser.get('plugins', 'to_load', [])
+        plugins.sort(key=lambda d: d.get(NAME, ''))
         for plugin in plugins:
             item = QListWidgetItem()
             item.setText(plugin[NAME])
             if plugin[MODULE_NAME] in to_load:
-                item.setCheckState(Qt.Checked)
+                item.setCheckState(Qt.CheckState.Checked)
             else:
-                item.setCheckState(Qt.Unchecked)
+                item.setCheckState(Qt.CheckState.Unchecked)
             item.plugin = plugin
             self._listbox.addItem(item)
-        
-        self.connect(self._listbox, 
-            SIGNAL('currentItemChanged(QListWidgetItem*, QListWidgetItem *)'),
+
+        self._listbox.currentItemChanged.connect(
             lambda item, previous: info_display.changeInfo(item.plugin))
-    
+
     def get_to_load(self):
         to_load = []
         for row in range(self._listbox.count()):
             item = self._listbox.item(row)
-            if item.checkState() == Qt.Checked:
+            if item.checkState() == Qt.CheckState.Checked:
                 to_load.append(item.plugin[MODULE_NAME])
         return to_load
-    
+
     def applySettings(self, control=None):
         to_load = self.get_to_load()
         cparser = PuddleConfig()
         cparser.set('plugins', 'to_load', to_load)
 
+
 if __name__ == '__main__':
     app = QApplication([])
     win = PluginConfig()
     win.show()
-    app.exec_()
\ No newline at end of file
+    app.exec()
